@@ -3,6 +3,8 @@ import pytest
 
 def test_import():
     from das import annot
+    from das import augmentation
+    from das import block_stratify
     from das import cli
     from das import data
     from das import data_hash
@@ -10,9 +12,11 @@ def test_import():
     from das import event_utils
     from das import io
     from das import make_dataset
+    from das import menagerie
     from das import models
     from das import tracking
     from das import npy_dir
+    from das import postprocessing
     from das import predict
     from das import pulse_utils
     from das import segment_utils
@@ -20,7 +24,5 @@ def test_import():
     from das import train_tune
     from das import utils
     from das import utils_plot
-    from das import augmentation
-    from das import block_stratify
     import das.kapre
     import das.tcn
